@@ -1,39 +1,40 @@
 pub fn main() {
     println!("cargo:rerun-if-changed=build.rs");
+    println!("cargo:rustc-check-cfg=cfg(coverage_nightly)");
     nightly();
     beta();
     stable();
-    msrv();
 }
 
 #[rustversion::nightly]
 fn nightly() {
+    println!("cargo:rustc-check-cfg=cfg(nightly)");
     println!("cargo:rustc-cfg=nightly");
 }
 
 #[rustversion::not(nightly)]
-fn nightly() {}
+fn nightly() {
+    println!("cargo:rustc-check-cfg=cfg(nightly)");
+}
 
 #[rustversion::beta]
 fn beta() {
+    println!("cargo:rustc-check-cfg=cfg(beta)");
     println!("cargo:rustc-cfg=beta");
 }
 
 #[rustversion::not(beta)]
-fn beta() {}
+fn beta() {
+    println!("cargo:rustc-check-cfg=cfg(beta)");
+}
 
 #[rustversion::stable]
 fn stable() {
+    println!("cargo:rustc-check-cfg=cfg(stable)");
     println!("cargo:rustc-cfg=stable");
 }
 
 #[rustversion::not(stable)]
-fn stable() {}
-
-#[rustversion::before(1.67)]
-fn msrv() {}
-
-#[rustversion::since(1.67)]
-fn msrv() {
-    println!("cargo:rustc-cfg=msrv");
+fn stable() {
+    println!("cargo:rustc-check-cfg=cfg(stable)");
 }
