@@ -6,16 +6,17 @@
 // option. All files in the project carrying such notice may not be copied,
 // modified, or distributed except according to those terms.
 
-#[cfg(feature = "gitcl")]
-pub(crate) mod cmd;
-#[cfg(feature = "git2")]
-pub(crate) mod git2;
-#[cfg(feature = "gix")]
-pub(crate) mod gix;
+use cfg_if::cfg_if;
 
-#[cfg(all(feature = "git", feature = "gitcl"))]
-pub(crate) use self::cmd::Config;
-#[cfg(all(feature = "git", feature = "git2"))]
-pub(crate) use self::git2::Config;
-#[cfg(all(feature = "git", feature = "gix"))]
-pub(crate) use self::gix::Config;
+cfg_if! {
+    if #[cfg(feature = "gitcl")] {
+        pub(crate) mod cmd;
+        pub(crate) use self::cmd::Config;
+    } else if #[cfg(feature = "git2")] {
+        pub(crate) mod git2;
+        pub(crate) use self::git2::Config;
+    } else {
+        pub(crate) mod gix;
+        pub(crate) use self::gix::Config;
+    }
+}
