@@ -17,13 +17,12 @@ use crate::{
 };
 use anyhow::{anyhow, Result};
 use std::env;
-use sysinfo::{
-    get_current_pid, CpuExt, Pid, Process, ProcessExt, System, SystemExt, User, UserExt,
-};
+use sysinfo::{get_current_pid, Cpu, Pid, Process, RefreshKind, System, User, Users};
 
 #[derive(Clone, Copy, Debug, Default)]
-#[allow(clippy::struct_excessive_bools)]
+#[allow(clippy::struct_excessive_bools, clippy::struct_field_names)]
 pub(crate) struct Config {
+    pub(crate) si_refresh_kind: Option<RefreshKind>,
     pub(crate) si_name: bool,
     pub(crate) si_os_version: bool,
     pub(crate) si_user: bool,
@@ -156,7 +155,8 @@ EmitBuilder::builder().idempotent().all_sysinfo().emit()?;
 impl EmitBuilder {
     /// Enable all of the `VERGEN_SYSINFO_*` options
     pub fn all_sysinfo(&mut self) -> &mut Self {
-        self.sysinfo_name()
+        self.sysinfo_refresh_kind(None)
+            .sysinfo_name()
             .sysinfo_os_version()
             .sysinfo_user()
             .sysinfo_memory()
@@ -167,6 +167,40 @@ impl EmitBuilder {
             .sysinfo_cpu_frequency()
     }
 
+    /// Set the [`RefreshKind`](sysinfo::RefreshKind) to use during sysinfo initialization.
+    ///
+    /// This allows the user to control at a more fine level what `sysinfo`
+    /// will refresh on initialization.
+    ///
+    /// # Example
+    /// ```
+    /// # use anyhow::Result;
+    /// # #[cfg(feature = "si")]
+    /// # use sysinfo::{CpuRefreshKind, RefreshKind};
+    /// # use vergen::EmitBuilder;
+    /// #
+    /// # pub fn main() -> Result<()> {
+    #[cfg_attr(
+        feature = "si",
+        doc = r##"
+let refresh_kind = RefreshKind::new();
+let cpu_refresh_kind = CpuRefreshKind::everything()
+    .without_cpu_usage()
+    .without_frequency();
+let config = EmitBuilder::builder()
+    .sysinfo_refresh_kind(Some(refresh_kind.with_cpu(cpu_refresh_kind)))
+    .sysinfo_cpu_brand()
+    .emit()?;
+"##
+    )]
+    /// #    Ok(())
+    /// # }
+    /// ```
+    pub fn sysinfo_refresh_kind(&mut self, refresh_kind: Option<RefreshKind>) -> &mut Self {
+        self.sysinfo_config.si_refresh_kind = refresh_kind;
+        self
+    }
+
     /// Enable the sysinfo name
     pub fn sysinfo_name(&mut self) -> &mut Self {
         self.sysinfo_config.si_name = true;
@@ -228,7 +262,7 @@ impl EmitBuilder {
         warnings: &mut Vec<String>,
     ) {
         if self.sysinfo_config.any() {
-            let system = setup_system();
+            let system = setup_system(self.sysinfo_config.si_refresh_kind);
 
             self.add_sysinfo_name(&system, idempotent, map, warnings);
             self.add_sysinfo_os_verison(&system, idempotent, map, warnings);
@@ -244,7 +278,7 @@ impl EmitBuilder {
 
     fn add_sysinfo_name(
         &self,
-        system: &System,
+        _system: &System,
         idempotent: bool,
         map: &mut RustcEnvMap,
         warnings: &mut Vec<String>,
@@ -256,7 +290,7 @@ impl EmitBuilder {
                 add_sysinfo_map_entry(
                     VergenKey::SysinfoName,
                     idempotent,
-                    system.name(),
+                    System::name(),
                     map,
                     warnings,
                 );
@@ -266,7 +300,7 @@ impl EmitBuilder {
 
     fn add_sysinfo_os_verison(
         &self,
-        system: &System,
+        _system: &System,
         idempotent: bool,
         map: &mut RustcEnvMap,
         warnings: &mut Vec<String>,
@@ -278,7 +312,7 @@ impl EmitBuilder {
                 add_sysinfo_map_entry(
                     VergenKey::SysinfoOsVersion,
                     idempotent,
-                    system.long_os_version(),
+                    System::long_os_version(),
                     map,
                     warnings,
                 );
@@ -395,7 +429,7 @@ impl EmitBuilder {
                         system
                             .cpus()
                             .iter()
-                            .map(CpuExt::name)
+                            .map(Cpu::name)
                             .collect::<Vec<&str>>()
                             .join(","),
                     ),
@@ -459,7 +493,8 @@ impl EmitBuilder {
     fn get_user(&self, system: &System) -> Option<String> {
         if let Ok(pid) = self.get_pid() {
             if let Some(process) = system.process(pid) {
-                for user in system.users() {
+                let users = Users::new_with_refreshed_list();
+                for user in &users {
                     if check_user(process, user) {
                         return Some(user.name().to_string());
                     }
@@ -501,11 +536,14 @@ fn add_sysinfo_map_entry(
     }
 }
 
-fn setup_system() -> System {
-    let mut system = System::new_all();
-    system.refresh_all();
-    system.refresh_users_list();
-    system
+fn setup_system(refresh_kind: Option<RefreshKind>) -> System {
+    if let Some(refresh_kind) = refresh_kind {
+        let mut system = System::new();
+        system.refresh_specifics(refresh_kind);
+        system
+    } else {
+        System::new_all()
+    }
 }
 
 fn check_user(process: &Process, user: &User) -> bool {
@@ -541,6 +579,7 @@ mod test {
     use crate::{emitter::test::count_idempotent, key::VergenKey, EmitBuilder};
     use anyhow::Result;
     use std::{collections::BTreeMap, env};
+    use sysinfo::{CpuRefreshKind, RefreshKind};
 
     const IDEM_COUNT: usize = 0;
     const SYSINFO_COUNT: usize = 9;
@@ -568,6 +607,23 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    #[serial_test::serial]
+    fn sysinfo_refresh_kind() -> Result<()> {
+        let refresh_kind = RefreshKind::new();
+        let cpu_refresh_kind = CpuRefreshKind::everything()
+            .without_cpu_usage()
+            .without_frequency();
+        let config = EmitBuilder::builder()
+            .sysinfo_refresh_kind(Some(refresh_kind.with_cpu(cpu_refresh_kind)))
+            .sysinfo_cpu_brand()
+            .test_emit()?;
+        assert_eq!(1, config.cargo_rustc_env_map.len());
+        assert_eq!(IDEM_COUNT, count_idempotent(&config.cargo_rustc_env_map));
+        assert_eq!(IDEM_COUNT, config.warnings.len());
+        Ok(())
+    }
+
     #[test]
     #[serial_test::serial]
     fn adding_none_defaults() -> Result<()> {
